<?php

namespace App\Services\Traits\Pdf;

use App\Support\Traits\Helpers;
use Dompdf\Dompdf;
use Dompdf\Options;

trait BasePdfTemplate
{
    use Helpers;

    private $pdf = null;

    public function getPdfApp(): Dompdf
    {
        if(!$this->pdf){
            $options = new Options();
            $options->setFontDir(app()->basePath('vendor/dompdf/dompdf/lib/fonts'));
            $options->setDefaultFont('微软雅黑');
            $options->setIsRemoteEnabled(true);
            $options->setDefaultPaperSize('A4' );
            $options->setDpi(168);
            $this->pdf = new Dompdf($options);
        }
        return $this->pdf;
    }

    /**
     * 添加合同logo和公章图片
     * @return void
     */
    public function addImage($pdfApp)
    {
        $canvas = $pdfApp->getCanvas();
        $w = $canvas->get_width();
        $h = $canvas->get_height();
        $y = $h -2 * 10 -24;

        $logoImage = storage_path('app/public/images/contract/logo.png');

        $canvas->page_script('
            $logoImage="'.$logoImage.'";
            $w="'.$w.'";
            $h="'.$h.'";
            $y="'.$y.'";
            $pdf->line(160, $y, $w-160, $y, "black",1);
            $pdf->set_opacity(1,"Multiply");
            $pdf->image($logoImage, 35, 55, 110, 42);
            '
        );
    }

    public function returnTemplateHtml($body): string
    {
        return <<<EOF
        <html lang="zh_CN no-js">
            <head>
            <meta http-equiv="Content-Type" content="text/html; charset=utf-8"/>
            <meta name="viewport" content="width=device-width, initial-scale=1.0, minimum-scale=1.0, maximum-scale=1.0, user-scalable=no">
            <style>
            body{  
                font-family: 'msyh'; 
                font-size: 0.8rem; 
                font-weight: bold; 
                display: block;
                word-wrap: break-word;
                overflow-wrap: break-word;
            }  
            
            .container {
                position: relative;
                z-index: 5;
                padding-right: 15px;
                padding-left: 15px;
                margin-right: auto;
                margin-left: auto;
                display: table;
                width: 100%;
            }
            
            .search-group {
                float: left;
                width: 100%;
                padding: 105px 0;
                font-size: 80px;
                font-weight: bold;
                display: table;
            }
            
            .logo-group {
                float: left;
                width: 260px;;
                height: 120px;
                display: table;
            }
            .order_title {
                float: left;
                width: 100%;
                height: 120px;
                display: table;
                text-indent: 10px; 
            }
          
            .itemTable {
                td {
                    height: 50px;
                    border: 1px solid #8a8888;
                    width: 100%;
                }
                .orderItemTd {
                    text-align: center;
                    background: #f2f2f2;
                }
                .orderItemTdContent {
                    text-align: center;
                }
            }
            
            .orderTable {
                td {
                   height: 50px; 
                } 
            }
            
            
            </style><title></title>
            </head><body><div class="container">$body</div></body>
        </html> 
        EOF;
    }

    public function outputPdf($html, $addLogoImage = true): ?string
    {
        if($addLogoImage){
            $this->addImage($this->getPdfApp());
        }
        $this->getPdfApp()->loadHTML($html, 'utf-8');
        $this->getPdfApp()->render();
        //$this->getPdfApp()->outputHtml();
        return $this->getPdfApp()->output();
    }
}
