<?php

namespace App\Support\Traits;

use Symfony\Component\HttpKernel\Exception\UnauthorizedHttpException;

trait Helpers
{
    // 取随机码，用于生成session
    protected function randomFromDevice($len)
    {
        $fp = @fopen('/dev/urandom', 'rb');
        $result = '';
       if ($fp !== FALSE) {
            $result .= @fread($fp, $len);
            @fclose($fp);
        }
        else
        {
            trigger_error('Can not open /dev/urandom.');
        }
        // convert from binary to string
        $result = base64_encode($result);
        // remove none url chars
        $result = strtr($result, '+/', '-_');
        return substr($result, 0, $len);
    }

    protected function generateToken($hashToken)
    {
        $expireAt = time() + 3600 * 24;
        $appToken = env('JWT_SECRET');

        $str = base64_encode(json_encode(['expire'=>$expireAt, 'hash'=>$hashToken])) ;
        $sign = md5($str . $appToken);

        return 'bearer'.base64_encode($sign . $str);
    }

    protected function decodeToken($token)
    {
        $str = base64_decode($token);
        $sign = substr($str, 0, 32);
        $dataStr = substr($str, 32);

        $appToken = env('JWT_SECRET');

        if($sign != md5($dataStr . $appToken))
        {
            throw new UnauthorizedHttpException("sign error");
        }
        $dataJson = base64_decode($dataStr);
        $data = json_decode($dataJson, 1);
        /*if($data['expire'] < time())
        {
            throw new UnauthorizedHttpException("token expired","身份验证失效，请重新登录");
        }*/
        return $data;
    }

    public static function printArr($arr){
        echo "<pre>";
        print_R($arr);
    }


    //驼峰命名转下划线命名
    protected function toUnderScore($str)
    {
        $dstr = preg_replace_callback('/([A-Z]+)/',function($matchs)
        {
            return '_'.strtolower($matchs[0]);
        },$str);
        return trim(preg_replace('/_{2,}/','_',$dstr),'_');
    }

    //下划线命名到驼峰命名
    protected function toCamelCase($str)
    {
        $array = explode('_', $str);
        $result = $array[0];
        $len=count($array);
        if($len>1)
        {
            for($i=1;$i<$len;$i++)
            {
                $result.= ucfirst($array[$i]);
            }
        }
        return $result;
    }

    /**
     * 将多维数组中的key转换类型
     * toCamelCase : 将下划线转换为驼峰格式 toUnderScore : 将驼峰转换为下划线分割
     * @param $arr
     * @param string $type
     * @return array
     */
    public function formatKeysfromArray($arr, $type = 'toCamelCase'){

        if(!is_array($arr)){
            $arr = $arr->toArray();
        }
        $newArr = [];
        collect($arr)->map(function($value, $key) use ( &$newArr, $type){
            if(!is_array($value)){
                switch($type) {
                    case 'toCamelCase' :
                        $newKey = $this->toCamelCase($key);
                        break;
                    case 'toUnderScore' :
                        $newKey = $this->toUnderScore($key);
                        break;
                    default:
                        $newKey = $this->toCamelCase($key);
                }
                $newArr[$newKey] = $value;
            }else{
                $val = $this->formatKeysfromArray($value,$type);
                $newArr[$key] = $val;
            }
        });

        return $newArr;
    }

    /**
     * 使用协程的方式返回大内容的csv数据
     * @param $file
     * @return bool|\Generator
     */
    public static function yieldLoadCsv($file,$maxLen = 99999999){
        if(!is_file($file)&&!file_exists($file)) {
            return false;
        }

        $cvs_file = fopen($file,'rb'); //开始读取csv文件数据
        while(feof($cvs_file) === false){
            yield fgetcsv($cvs_file);
        }
        fclose($cvs_file);
    }

    function charsetToGBK($mixed)
    {
        if (is_array($mixed)) {
            foreach ($mixed as $k => $v) {
                if (is_array($v)) {
                    $mixed[$k] = $this->charsetToGBK($v);
                } else {
                    $encode = mb_detect_encoding($v, array('ASCII', 'UTF-8', 'GB2312', 'GBK', 'BIG5'));
                    if ($encode == 'UTF-8') {
                        $mixed[$k] = iconv('UTF-8', 'GBK', $v);
                    }
                }
            }
        } else {
            $encode = mb_detect_encoding($mixed, array('ASCII', 'UTF-8', 'GB2312', 'GBK', 'BIG5'));
            if ($encode == 'UTF-8') {
                $mixed = iconv('UTF-8', 'GBK', $mixed);
            }
        }
        return $mixed;
    }

    function stripHtml($str): string
    {
        $str = str_replace("<br>", "", $str);
        $str = str_replace("\t", "", $str);
        //$str = htmlspecialchars($str);
        $str = htmlspecialchars_decode($str);
        return strip_tags($str);
    }

    function stripTabTag($str)
    {
        $str = str_replace("\t", "", $str);
        $str = str_replace("\r\n", "", $str);
        return $str;
    }

    public function validMobile($mobile)
    {
        return preg_match("/^1[3456789][0-9]{9}$/", $mobile);
    }

    public function validEmail($email)
    {
        return preg_match("/^\w+([-+.]\w+)*@\w+([-.]\w+)*\.\w+([-.]\w+)*$/", $email);
    }

}
